@@ -16,7 +16,7 @@ from .views import login, reset_cookie
 import datetime
 from .Common import get_redirect_url
 from django.utils import timezone
-from django.http import JsonResponse
+
 
 def recheck(request):
     """Starts FIDO2 recheck"""
@@ -49,15 +49,13 @@ def begin_registeration(request):
 def complete_reg(request):
     """Completes the registeration, called by API"""
     try:
-        if not "fido_state" in request.session:
-            return JsonResponse({'status': 'ERR', "message": "FIDO Status can't be found, please try again"})
         data = cbor.decode(request.body)
 
         client_data = CollectedClientData(data['clientDataJSON'])
         att_obj = AttestationObject((data['attestationObject']))
         server = getServer()
         auth_data = server.register_complete(
-            request.session.pop('fido_state'),
+            request.session['fido_state'],
             client_data,
             att_obj
         )
@@ -68,7 +66,11 @@ def complete_reg(request):
         uk.owned_by_enterprise = getattr(settings, "MFA_OWNED_BY_ENTERPRISE", False)
         uk.key_type = "FIDO2"
         uk.save()
-        return HttpResponse(simplejson.dumps({'status': 'OK'}))
+        if getattr(settings, 'MFA_ENFORCE_RECOVERY_METHOD', False) and not User_Keys.objects.filter(key_type = "RECOVERY", username=request.user.username).exists():
+            request.session["mfa_reg"] = {"method":"FIDO2","name": getattr(settings, "MFA_RENAME_METHODS", {}).get("FIDO2", "FIDO2")}
+            return HttpResponse(simplejson.dumps({'status': 'RECOVERY'}))
+        else:
+            return HttpResponse(simplejson.dumps({'status': 'OK'}))
     except Exception as exp:
         import traceback
         print(traceback.format_exc())
@@ -77,13 +79,15 @@ def complete_reg(request):
             client.captureException()
         except:
             pass
-        return JsonResponse({'status': 'ERR', "message": "Error on server, please try again later"})
+        return HttpResponse(simplejson.dumps({'status': 'ERR', "message": "Error on server, please try again later"}))
 
 
 def start(request):
-    """Start Registeration a new FIDO Token"""
+    """Start Registration a new FIDO Token"""
     context = csrf(request)
     context.update(get_redirect_url())
+    context["method"] = {"name":getattr(settings,"MFA_RENAME_METHODS",{}).get("FIDO2","FIDO2 Security Key")}
+    context["RECOVERY_METHOD"]=getattr(settings,"MFA_RENAME_METHODS",{}).get("RECOVERY","Recovery codes")
     return render(request, "FIDO2/Add.html", context)
 
 
@@ -139,7 +143,7 @@ def authenticate_complete(request):
             except:
                 pass
             return HttpResponse(simplejson.dumps({'status': "ERR",
-                                                  "message": excep.message}),
+                                                  "message": str(excep)}),
                                 content_type = "application/json")
 
         if request.session.get("mfa_recheck", False):
